﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Collections;
using System.Threading;

namespace RinGOch
{
	public class WebCacheReader
	{
		#region メンバ変数
		public static string CacheDirectory = "";
		#endregion

		#region パスを取得
		static void CheckDirectoryExists(Guid id)
		{
			string path = Path.Combine(CacheDirectory,id.ToString());
			if(!Directory.Exists(path))
				Directory.CreateDirectory(path);
		}
		static string GetHeaderFile(Guid id)
		{
			return Path.Combine(CacheDirectory,id.ToString() + "\\Header.dat");
		}
		static string GetHashFile(WebHeaderInfo info)
		{
			return Path.Combine(CacheDirectory,info.GUID.ToString() + "\\Auxiliary2.dat");
		}
		static string GetGuidFile(WebHeaderInfo info)
		{
			return Path.Combine(CacheDirectory,info.GUID.ToString() + "\\Auxiliary1.dat");
		}
		static string GetPostFile(WebHeaderInfo info)
		{
			return Path.Combine(CacheDirectory,info.GUID.ToString() + "\\Posts.dat");
		}
		#endregion

		#region キャッシュ操作

		#region キャッシュを持っているかチェック
		public static bool IsCached(Guid id)
		{
			return Directory.Exists(Path.Combine(CacheDirectory,id.ToString()));
		}
		public static bool IsCached(WebHeaderInfo info)
		{
			return IsCached(info.GUID);
		}
		#endregion

		#region 発言GUIDを読み取り
		public static Guid[] ReadPostGuids(Guid id)
		{
			try
			{
				Interface.LockWebCache(id);

				// 存在を確認
				CheckDirectoryExists(id);

				// ヘッダ読み取り
				WebHeaderInfo info = ReadHeaderFromFile(id,false);
			
				// ポストファイルを開く
				using(FileStream fs = new FileStream(GetGuidFile(info),FileMode.OpenOrCreate,FileAccess.Read,FileShare.Read))
				{
					int v;
					int posts = (int)fs.Length / (16 + 4);
					Guid[] guids = new Guid[posts];
					using(BinaryReader reader = new BinaryReader(fs))
					{
						for(int i=0;i<posts;i++)
						{
							guids[i] = new Guid(reader.ReadBytes(16));
							v = reader.ReadInt32();
						}
					}
					fs.Close();
					return guids;
				}
			}
			finally
			{
				Interface.UnlockWebCache(id);
			}
		}
		#endregion

		#region すべてのPOSTを読み取り
		public static WebPostInfo[] ReadAllPost(Guid id)
		{
			try
			{
				Interface.LockWebCache(id);

				// Directoryの存在を確認
				CheckDirectoryExists(id);

				// ヘッダ読み取り
				WebHeaderInfo info = ReadHeaderFromFile(id,false);
			
				// 配列確保
				WebPostInfo[] posts = new WebPostInfo[info.Posts];

				// ポストファイルを開く
				using(FileStream fs = new FileStream(GetPostFile(info),FileMode.OpenOrCreate,FileAccess.Read,FileShare.Read))
				{
					for(int i=0;i<posts.Length;i++)
						posts[i] = (WebPostInfo)BinaryCoder.Deserialize(fs);
				}

				return posts;
			}
			finally
			{
				Interface.UnlockWebCache(id);
			}
		}
		#endregion

		#region GUIDで指定されたPOSTを読み取り
		public static WebPostInfo[] ReadAllPostusingGuid(Guid id,Guid[] guids)
		{
			try
			{
				Interface.LockWebCache(id);

				// Directoryの存在を確認
				CheckDirectoryExists(id);

				// ヘッダ読み取り
				WebHeaderInfo info = ReadHeaderFromFile(id,false);
			
				// 配列確保
				ArrayList list = new ArrayList();

				// ポストファイルを開く
				using(FileStream fs = new FileStream(GetPostFile(info),FileMode.OpenOrCreate,FileAccess.Read,FileShare.Read))
				{
					for(int i=0;i<info.Posts;i++)
					{
						WebPostInfo post = (WebPostInfo)BinaryCoder.Deserialize(fs);
						if(Array.IndexOf(guids,post.GUID) >= 0)
							list.Add(post);
					}
				}

				return (WebPostInfo[])list.ToArray(typeof(WebPostInfo));
			}
			finally
			{
				Interface.UnlockWebCache(id);
			}
		}
		#endregion

		#endregion

		#region ヘッダ読み取り
		public static WebHeaderInfo ReadHeader(Guid id)
		{
			return ReadHeaderFromFile(id);
		}
		public static WebHeaderInfo ReadHeaderFromFile(Guid id)
		{
			return ReadHeaderFromFile(id,true);
		}
		public static WebHeaderInfo ReadHeaderFromFile(Guid id,bool block)
		{
			try
			{
				if(block) Interface.LockWebCache(id);

				// Directoryの存在を確認
				CheckDirectoryExists(id);

				using(FileStream fs = new FileStream(GetHeaderFile(id),FileMode.Open,FileAccess.Read,FileShare.Read))
				{
					WebHeaderInfo head = (WebHeaderInfo)BinaryCoder.Deserialize(fs);
					fs.Close();
					return head;
				}
			}
			catch
			{
				return null;
			}
			finally
			{
				if(block) Interface.UnlockWebCache(id);
			}
		}
		#endregion
	}
}
